/************************************************************************************/
/*                     Electronic Software Distribution                             */
/* FILENAME       :  CURDEMO.REX                                                    */
/* TYPE           :  IBM ClassicRexx - PC DOS 7 or PC DOS 2000                      */
/* DESCRIPTION    :  This module shows the features of CUR on DOS                   */
/*                                                                                  */
/* AUTHOR         :  Michael D Lueck                                                */
/*                   mdlueck@lueckdatasystems.com                                   */
/*                                                                                  */
/* NEEDS          :                                                                 */
/*                                                                                  */
/* USEAGE         :  [/d] [/v] [/??] [/?]                                           */
/*                                                                                  */
/* REVISION HISTORY                                                                 */
/*                                                                                  */
/* DATE       REVISED BY DESCRIPTION OF CHANGE                                      */
/* ---------- ---------- -------------------------------------------------------    */
/* 01/31/2000 MDL        Initial Creation, based on CUR.CMD v1 by WRL               */
/* 07/21/2000 MDL        Initial Creation of this Demo Program                      */
/*                        Changed CUR API's to accept some params vs the global     */
/*                        vars                                                      */
/*                        Allowed for multiple FINDarray stems, other vars are      */
/*                        still shared global vars so be carefull                   */
/* 08/02/2000 MDL        Ported Logger procedure                                    */
/* 08/26/2000 MDL        Ported COPYfile procedure                                  */
/* 08/26/2000 MDL        Performance testing, polishing, cross platform sync with   */
/*                        other CUR source files                                    */
/* 08/29/2000 MDL        Added SORTarray                                            */
/* 08/31/2000 MDL        Added LDSFileSpec and defaulting ??RXLOGfilename           */
/* 09/01/2000 MDL        Added ??RXTEMPdir                                          */
/* 09/01/2000 MDL        Added INDEXarray, added better error recovery to WRITEfile */
/* 09/01/2000 MDL        Release of version 3.0.0, FINALLY!!!                       */
/************************************************************************************/

/************************************************************************************/
/*__________________________________________________________________________________*/
/*                    The Beginning of the Main Routine.                            */

/* Process the command line paramaters */
parse upper arg uparg
if \ PARMcheck(uparg) then exit 1
drop uparg

/* Demo Starts Here */

say 'Test READfile'
if READfile('MYstem', 'C:\Config.sys') then do
   say 'This is back in the real rexx code again'
      do i=1 to MYstem.0
      say MYstem.i
   end
end

say 'Test SEARCHline'
FOUNDline=SEARCHline('MYstem', 'FILES=', 1, 'I')
say 'SEARCHline found the string on line: 'FOUNDline' -> 'MYstem.FOUNDline

FOUNDline=SEARCHline('MYstem', 'BUFFERS=', 1, '')
say 'SEARCHline found the string on line: 'FOUNDline' -> 'MYstem.FOUNDline

say 'Test INSERTline'
if INSERTline('MYstem', '***THIS IS MY NEW LINE***', 4) then say 'INSERTline worked'
if INSERTline('MYstem', '***YOU WILL NOT SEE THIS EVER***', MYstem.0+4) then say 'INSERTline is brocken!!!!'

say 'Test APPENDline'
rc=APPENDline('MYstem', '***THIS WILL END UP AT THE END OF THE FILE***')

say 'Test DELline'
if DELline('MYstem', 6) then say 'DELline nuked line 6'
if DELline('MYstem', MYstem.0+4) then say 'DELline is brocken!!!!'

say 'Test WRITEfile'
/* Usually if we are in DEBUG mode, then we don't want to write out our changes */
if DEBUG=0 then if WRITEfile('MYstem', 'trash') then say 'WRITEfile ran OK'
else say 'WRITEfile had an error'

say 'Test INSERTtext'
mystring='C:\1;C:\2;C:\3;C:\4;'

say mystring
say 'Tests where the FINDtext is found'
mystring=INSERTtext(mystring, 'C:\3;', 'C:\5;', 'I', 'A')
say mystring
mystring=INSERTtext(mystring, 'C:\2;', 'C:\6;', 'I', 'B')
say mystring
say 'Tests where the FINDtext is not found'
mystring=INSERTtext(mystring, 'the;', 'C:\7;', 'I', 'A')
say mystring
mystring=INSERTtext(mystring, 'the;', 'C:\8;', 'I', 'B')
say mystring

say 'Test REPLACEtext'
mystring='This is a hard test that needs a lot of testing!'

say mystring
mystring=REPLACEtext(mystring, 'a', 'GEE', '')
say mystring

say 'Testing COPYfile'
rc=LineOut('ZERO')
if COPYfile('ZERO', 'ONE') then do
   say 'COPYfile works!'
end
else do
   say 'COPYfile failed!'
end

say 'Test Logger'
if Logger(??RXLOGfilename, 'This is a sample log entry with time stamp.', 1) then do
   say 'Logger works!'
end
else do
   say 'Logger failed!'
end

say 'Test ISpctag'
if ISpctag('PC12345') then do
   say 'It is a PCTag'
end
else do
   say 'It is not a PCTag'
end

say 'Testing LDSFileSpec'
say 'DOS Mode Test'
say 'String: '??RXFileName
say 'Drive:  'LDSFileSpec('DRIVE', ??RXFileName)
say 'Path:   'LDSFileSpec('PATH', ??RXFileName)
say 'Name:   'LDSFileSpec('NAME', ??RXFileName)
say ''

MYunc='\\LDSWS01\DATA\DEV\REXX\TEST.CMD'
say 'UNC Mode Test'
say 'String: 'MYunc
say 'Drive:  'LDSFileSpec('DRIVE', MYunc)
say 'Path:   'LDSFileSpec('PATH', MYunc)
say 'Name:   'LDSFileSpec('NAME', MYunc)
say ''

say 'LDSFILEspec in action!'
say ??RXLOGfilename

say 'Testing INDEXarray'
STATEarray.1='MI'
STATEarray.2='ME'
STATEarray.3='FL'
STATEarray.0=3

if INDEXarray('STATEarray', 'STATEindex') then do
   say 'INDEXarray worked, so now we will test the index it built'
   THISstate='MI'
   if STATEindex.THISstate \= 0 then say 'We have state: 'THISstate
   else say 'We do not have state: 'THISstate
   THISstate='CO'
   if STATEindex.THISstate \= 0 then say 'We have state: 'THISstate
   else say 'We do not have state: 'THISstate
end

say 'Testing SORTarray'
if SORTarray('STATEarray') then do 
   say 'SORTarray worked!'
   do i=1 to STATEarray.0
      say STATEarray.i
   end
end
else say 'SORTarray failed!'

/* Demo Ends Here */

exit 0
/*         The end of the main routine returning to the operating system...         */
/*__________________________________________________________________________________*/
/************************************************************************************/

/************************************************************************************/
/*__________________________________________________________________________________*/
PARMCHECK:
??uparg=arg(1)

if wordpos('/??',??uparg)\=0 then do; call helpparms2; return 0; end;
if wordpos('/?',??uparg)\=0 then do; call helpparms; return 0; end;

if wordpos('/D',??uparg)\=0 then debug=1
else debug=0
if wordpos('/V',??uparg)\=0 then verbose=1
else verbose=0

/* Parse the SOURCE to get various informations */
parse source ??RXos ??RXFileType ??RXFileName

/* Define a default ??RXLOGfilename for this program, for use with Logger */
??RXLOGfilename=LDSFileSpec('DRIVE', ??RXFileName)
??RXLOGfilename=??RXLOGfilename || LDSFileSpec('PATH', ??RXFileName)
??RXLOGfilename=??RXLOGfilename || Left(LDSFileSpec('NAME', ??RXFileName), LastPos('.', LDSFileSpec('NAME', ??RXFileName))-1) || '.log'

/* Define ??RXTEMPdir to point to the OS temp dir */
??temp=value('TEMP',,'DOSENVIRONMENT')
??tmp=value('TMP',,'DOSENVIRONMENT')
select
   when ISdir(??temp) then do
      ??RXTEMPdir=??temp
   end
   when ISdir(??tmp) then do
      ??RXTEMPdir=??tmp
   end
   otherwise do
      ??RXTEMPdir=SysBootDrive() || '\Temp'
      if \ ISdir(??RXTEMPdir) then do
         if \ MAKEdir(??RXTEMPdir) then do
            say 'Error! -> Unable to create temp directory'
            return 0
         end
      end
   end
end /* select */

/* PROGRAM PARMS START HERE - DO NOT REMOVE THIS LINE */
/* PROGRAM PARMS END HERE - DO NOT REMOVE THIS LINE */

drop ??uparg ??temp ??tmp

return 1
/*__________________________________________________________________________________*/
/************************************************************************************/

/************************************************************************************/
/*__________________________________________________________________________________*/
HELPPARMS:
say '_____________________'
say 'Use CURDEMO.rex exec as a template to build quick Rexx execs to update files'
say 'Syntax:'
say '  CURDEMO [/d] [/v] [/??] [/?]'
say 'Where:'
say ' /d           Specifies a debug run.'
say ' /v           Specifies a verbose run.  Will display run status'
say ' /??          Specifies extended help: displays what is updated'
say ' /?           Specifies to display this text'
return
/*__________________________________________________________________________________*/
/************************************************************************************/

/************************************************************************************/
/*__________________________________________________________________________________*/
HELPPARMS2:
say '_____________________'
say 'CURDEMO extended help'
say ''
say ' In this area you add what it is that'
say ' the exec changes in the files'

return
/*__________________________________________________________________________________*/
/************************************************************************************/

/* <--PCDOSREXXCUR DO NOT REMOVE THIS FLAG LINE--> */
/* Version 3.0.0 */
/******************************************************************************************/
/* The following routines are provided to enable more efficient changes of system files.  */
/* The routines are not proceduralized yet so one must set the following variables prior  */
/* to call.  Documentation is forth coming. WRL                                           */
/******************************************************************************************/
/* Well, 5 years later, this code has been ported from two CUR versions on OS/2 to this   */
/* version for IBM's ClassicRexx found in PC DOS 7 and 2000. Some API's are now           */
/* proceduralized, all vars starting with '??' are reserved for CUR use, and many API's   */
/* are new to this version of the code, which will also appear in the other various ports */
/* I create. Happy Coding! MDL                                                            */
/******************************************************************************************/
/* READfile    FINDarray FINDfilename           modifies FINDarray and returns logical rc */
/* WRITEfile   FINDarray FINDfilename                                  returns logical rc */
/* INSERTtext  FINDstring FINDtext PUTtext FINDcase INSERTmode         returns FINDstring */
/* REPLACEtext FINDstring FINDtext PUTtext [FINDcase]                  returns FINDstring */
/* LOWERtext   FINDstring                                              returns FINDstring */
/* SEARCHline  FINDarray FINDstring FINDstartline [FINDcase]           returns FINDstring */
/* INSERTline  FINDarray FINDstring FINDline                           returns logical rc */
/* APPENDline  FINDarray FINDstring                                    returns logical rc */
/* DELline     FINDarray FINDline                                      returns logical rc */
/* INDEXarray  FINDarray FINDindex [FINDcase]   modifies FINDindex and returns logical rc */
/* SORTarray   FINDarray [UPdown]               modifies FINDarray and returns logical rc */
/* DELfile     FINDfilename                                            returns logical rc */
/* COPYfile    FROMfilename TOfilename                                 returns logical rc */
/* MAKEdir     FINDfilename                                            returns logical rc */
/* DELdir      FINDfilename                                            returns logical rc */
/* ISdir       FINDfilename                                            returns logical rc */
/* ISfile      FINDfilename                                            returns logical rc */
/* LDSFileSpec RUNmode FINDfilename                                    returns FINDstring */
/* Logger      LOGfilename LOGstring [TimeStampEntry]                  returns logical rc */
/* ISpctag     PCTag                                                   returns logical rc */
/* SETUPcolor  Creates global stem COLORarray                          returns logical rc */
/******************************************************************************************/

/*********************/
READfile:
??FINDarray=arg(1)
??FINDfilename=arg(2)

if VERBOSE then say 'In READfile, ??FINDarray='??FINDarray' ??FINDfilename='??FINDfilename

/* Check Params */
if strip(??FINDarray)='' | strip(??FINDfilename)='' then do
   say 'READfile ERROR! -> Not Enough Paramaters'
   return 0
end

if \ ISfile(??FINDfilename) then do
   say 'The file that was to be read '??FINDfilename' is not on this machine'
   return 0
end

/* Reset vars in case we are using a stem over again */
??FOUNDline=0;
interpret 'drop '??FINDarray'.; '??FINDarray'.=0'

/* Open a stream to read the file */
??rc=stream(??FINDfilename,'C','OPEN READ')
if ??rc\='READY' then do
   say 'READfile -> Can not open the file in READ mode, ??RC='??RC
   return 0
end

/* Read in the file to the array */
do while lines(??FINDfilename)
   ??FOUNDline=??FOUNDline+1
   ??trash=value(??FINDarray||'.'||??FOUNDline, LineIn(??FINDfilename))
   if VERBOSE then say 'line '??foundline'='value(??FINDarray||'.'||??FOUNDline)
end
??trash=value(??FINDarray||'.0', ??FOUNDline)
drop ??trash

/* Close the stream */
??rc=stream(??FINDfilename,'C','CLOSE')

if VERBOSE then say 'fileread in elements='value(??FINDarray||'.0')
if VERBOSE then say 'Last Line='value(??FINDarray||'.'||value(??FINDarray||'.0'))

/* Global Var Cleanup */
drop ??FINDarray ??FINDfilename ??rc ??FOUNDline

return 1

/*********************/
WRITEfile:
??FINDarray=arg(1)
??FINDfilename=arg(2)

if VERBOSE then say 'In WRITEfile, '??FINDarray'.0='value(??FINDarray||'.0') ??FINDarray'.1='value(??FINDarray||'.1') '??FINDfilename='??FINDfilename

/* Check Params */
if strip(??FINDarray)='' | strip(??FINDfilename)='' then do
   say 'WRITEfile ERROR! -> Not Enough Paramaters'
   return 0
end

/* Check that we have a good FINDarray to work with */
if value(??FINDarray||'.0')=0 | \ DataType(value(??FINDarray||'.0'), 'N') then do
   say 'WRITEfile ERROR! -> FINDarray='??FINDarray' not in proper format'
   return 0
end

/* Delete the file we are writing out if it exists */
if ISfile(??FINDfilename) then do
   if \ DELfile(??FINDfilename) then do
      say 'Unable to delete the file'??FINDfilename
      return 0
   end
end

/* Open a stream to read the file */
??rc=stream(??FINDfilename,'C','OPEN WRITE')
if ??rc\='READY' then do
   say 'WRITEfile -> Can not open the file in WRITE mode, ??RC='??RC
   return 0
end

/* Write out the file */
do ??i=1 to value(??FINDarray||'.0')
   if VERBOSE then say 'FINDarray.??i='value(??FINDarray||'.'||??i)
   ??rc=LineOut(??FINDfilename, value(??FINDarray||'.'||??i))
   if ??rc\=0 then do
      say 'WRITEfile -> Error calling LineOut, ??RC='??RC', '??FINDarray'.'??i'='value(??FINDarray||'.'||??i)
      return 0
   end
end

/* Close the stream */
??rc=stream(??FINDfilename,'C','CLOSE')

/* Global Var Cleanup */
drop ??FINDarray ??FINDfilename ??rc ??i

return 1

/*********************/
INSERTtext:
??FINDstring=arg(1)
??FINDtext=arg(2)
??PUTtext=arg(3)
??FINDcase=arg(4)
??INSERTmode=arg(5)

if VERBOSE then say 'In INSERTtext, ??FINDstring='??FINDstring' ??PUTtext='??PUTtext' ??FINDtext='??FINDtext' ??FINDcase='??FINDcase

/* Check Params */
if strip(??FINDstring)='' | strip(??FINDtext)='' | strip(??PUTtext)='' then do
   say 'INSERTtext ERROR! -> Not Enough Paramaters'
   return ??FINDstring
end

??FINDcase=translate(??FINDcase)
??INSERTmode=translate(??INSERTmode)

/* Make sure we are set to 'B'efore or 'A'fter mode */
if ??INSERTmode='B' | ??INSERTmode='A' then do
   /* 'I' indicates to be Insensative to case, so everything must go to upper case */
   if ??FINDcase='I' then do;??FINDstring=translate(??FINDstring);??PUTtext=translate(??PUTtext);??FINDtext=translate(??FINDtext);end

   /* Only continue if we do not find the PUTtext currently in the string */
   if pos(??PUTtext, ??FINDstring)=0 then do
      /* Only Insert if we find the FINDtest in the FINDstring... */
      if pos(??FINDtext,??FINDstring)\=0 then do
         select
            when ??INSERTmode='B' then do
               ??FINDstring=insert(??PUTtext, ??FINDstring, pos(??FINDtext, ??FINDstring)-1)
            end
            when ??INSERTmode='A' then do
               ??FINDstring=insert(??PUTtext, ??FINDstring, pos(??FINDtext, ??FINDstring)+length(??FINDtext)-1)
            end
         end /* select */
      end /*if */
      /* Else use INSERTmode to Prefix or Append the FINDstring */
      else do
         select
            when ??INSERTmode='B' then do
               ??FINDstring=??PUTtext || ??FINDstring
            end
            when ??INSERTmode='A' then do
               ??FINDstring=??FINDstring || ??PUTtext
            end
         end /* select */
      end /* else */
      if VERBOSE then say 'INSERTtext -> New ??FINDstring='??FINDstring
   end
end
else do
   say 'INSERTtext ERROR! -> ??INSERTmode has an invalid value, ??INSERTmode='??INSERTmode
end

/* Global Var Cleanup */
drop ??PUTtext  ??FINDtext ??FINDcase ??INSERTmode

return ??FINDstring

/*********************/
REPLACEtext:
??FINDstring=arg(1)
??FINDtext=arg(2)
??PUTtext=arg(3)
??FINDcase=arg(4)

if VERBOSE then say 'In REPLACEtext, ??FINDstring='??FINDstring' ??PUTtext='??PUTtext' ??FINDtext='??FINDtext' ??FINDcase='??FINDcase

/* Check Params */
if strip(??FINDstring)='' | strip(??FINDtext)='' | strip(??PUTtext)='' then do
   say 'REPLACEtext ERROR! -> Not Enough Paramaters'
   return ??FINDstring
end

/* 'I' indicates to be Insensative to case, so everything must go to upper case */
if translate(??FINDcase)='I' then do;??FINDstring=translate(??FINDstring);??PUTtext=translate(??PUTtext);??FINDtext=translate(??FINDtext);end

/* In memory of Steve, first see if the ??FINDtext is in the ??FINDstring so we don't screw up */
/* If there is work to be done, then do it until it is done */
if pos(??FINDtext,??FINDstring)\=0 then do until pos(??FINDtext,??FINDstring)=0
    ??FINDstring=substr(??FINDstring, 1, pos(??FINDtext, ??FINDstring)-1) || ??PUTtext || substr(??FINDstring, pos(??FINDtext, ??FINDstring)+length(??FINDtext))
    if VERBOSE then say ??FINDstring
end

/* Global Var Cleanup */
drop ??PUTtext  ??FINDtext ??FINDcase

return ??FINDstring

/*********************/
LOWERtext: procedure expose verbose
FINDstring=arg(1)

if VERBOSE then say 'In LOWERline, FINDstring='FINDstring

/* Check Params */
if strip(??FINDarray)='' | strip(??FINDstring)='' | strip(??FINDstartline)='' then do
   say 'LOWERtext ERROR! -> Not Enough Paramaters'
   return FINDstring
end

FINDstring=translate(FINDstring, 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')

return FINDstring

/*********************/
SEARCHline:
??FINDarray=arg(1)
??FINDstring=arg(2)
??FINDstartline=arg(3)
??FINDcase=arg(4)

if VERBOSE then say 'In SEARCHline, ??FINDarray='??FINDarray' ??FINDstring='??FINDstring' ??FINDstartline='??FINDstartline' ??FINDcase='??FINDcase

/* Check Params */
if strip(??FINDarray)='' | strip(??FINDstring)='' | strip(??FINDstartline)='' then do
   say 'SEARCHline ERROR! -> Not Enough Paramaters'
   return 0
end

??FOUNDline=0

/* Case insensitive mode */
if translate(??FINDcase)='I' then do
   ??FINDstring=strip(translate(??FINDstring))

   do ??line=??FINDstartline to value(??FINDarray'.0')
      ??thisline=translate(value(??FINDarray'.'??line))
      if VERBOSE then say 'line='??line ' thisline='??thisline
      if pos(??FINDstring,??thisline)\=0 then do;??FOUNDline=??line; leave; end
   end
end
/* Case sensitive mode */
else do
   ??FINDstring=strip(??FINDstring)

   do ??line=??FINDstartline to value(??FINDarray'.0')
      ??thisline=value(??FINDarray'.'??line)
      if VERBOSE then say 'line='??line ' thisline='??thisline
      if pos(??FINDstring,??thisline)\=0 then do;??FOUNDline=??line; leave; end
   end
end

/* Global Var Cleanup */
drop ??FINDarray ??FINDstring ??FINDstartline ??FINDcase

return ??FOUNDline

/*********************/
INSERTline:
??FINDarray=arg(1)
??FINDstring=arg(2)
??FINDline=arg(3)

if VERBOSE then say 'In INSERTline, ??FINDarray='??FINDarray' ??FINDstring='??FINDstring' ??FINDline='??FINDline

/* Check Params */
if strip(??FINDarray)='' | strip(??FINDstring)='' | strip(??FINDline)='' then do
   say 'INSERTline ERROR! -> Not Enough Paramaters'
   return 0
end

if ??FINDline<value(??FINDarray'.0') then do
   do ??line=value(??FINDarray'.0') to ??FINDline by -1
      ??linep1=??line+1
      if VERBOSE then say 'old='value(??FINDarray'.'??linep1)' new='value(??FINDarray'.'??line)
      ??trash=value(??FINDarray'.'??linep1, value(??FINDarray'.'??line))
   end /*do */
   ??trash=value(??FINDarray'.'??FINDline, ??FINDstring)
   ??trash=value(??FINDarray'.0', value(??FINDarray'.0')+1)
end
else do
   say 'INSERTline ERROR! -> ??FINDline='??FINDline' is beyond the end of ??FINDarray.0='value(??FINDarray'.0')
   return 0
end

/* Global Var Cleanup */
drop ??FINDarray ??FINDstring ??FINDline ??trash

return 1

/*********************/
APPENDline:
??FINDarray=arg(1)
??FINDstring=arg(2)

if VERBOSE then say 'In APPENDline, ??FINDarray='??FINDarray' ??FINDstring='??FINDstring

/* Check Params */
if strip(??FINDarray)='' | strip(FINDstring)='' then do
   say 'APPENDline ERROR! -> Not Enough Paramaters'
   return 0
end

??newline=value(??FINDarray'.0')+1
??trash=value(??FINDarray'.'??newline, ??FINDstring)
??trash=value(??FINDarray'.0', ??newline)

/* Global Var Cleanup */
drop ??FINDarray ??FINDstring ??newline ??trash

return 1

/*********************/
DELline:
??FINDarray=arg(1)
??FINDline=arg(2)

if VERBOSE then say 'In DELline, ??FINDarray='??FINDarray' ??FINDline='??FINDline

/* Check Params */
if strip(??FINDarray)='' | strip(??FINDline)='' then do
   say 'DELline ERROR! -> Not Enough Paramaters'
   return 0
end

if ??FINDline<value(??FINDarray'.0') then do
   do ??line=??FINDline+1 to value(??FINDarray'.0')
      ??linep1=??line-1
      if VERBOSE then say 'old='value(??FINDarray'.'??linep1)' new='value(??FINDarray'.'??line)
      ??trash=value(??FINDarray'.'??linep1, value(??FINDarray'.'??line))
   end /*do */
   ??line=value(??FINDarray'.0')
   interpret 'drop '??FINDarray'.'??line
   ??trash=value(??FINDarray'.0', value(??FINDarray'.0')-1)
end
else do
   say 'DELline ERROR! -> ??FINDline='??FINDline' is beyond the end of ??FINDarray.0='value(??FINDarray'.0')
   return 0
end

/* Global Var Cleanup */
drop ??FINDarray ??FINDline ??line ??trash

return 1

/*********************/
INDEXarray:
??FINDarray=arg(1)
??FINDindex=arg(2)
??FINDcase=arg(3)

if VERBOSE then say 'In INDEXarray, ??FINDarray='??FINDarray' ??FINDindex='??FINDindex' ??FINDcase='??FINDcase

/* Check Params */
if strip(??FINDarray)='' | strip(??FINDindex)='' then do
   say 'INDEXarray ERROR! -> Not Enough Paramaters'
   return 0
end

/* Check that we have a good FINDarray to work with */
if value(??FINDarray||'.0')=0 | \ DataType(value(??FINDarray||'.0'), 'N') then do
   say 'INDEXarray ERROR! -> FINDarray='??FINDarray' not in proper format'
   return 0
end

/* Reset vars in case we are using a stem over again */
interpret 'drop '??FINDindex'.; '??FINDindex'.=0'

/* Case insensitive mode */
if translate(??FINDcase)='I' then do
   do ??i=1 to value(??FINDarray||'.0')
      ??THISline=translate(value(??FINDarray||'.'||??i))
      ??trash=value(??FINDindex||'.'||??THISline, ??i)
   end
end
/* Case sensitive mode */
else do
   do ??i=1 to value(??FINDarray||'.0')
      ??THISline=value(??FINDarray||'.'||??i)
      ??trash=value(??FINDindex||'.'||??THISline, ??i)
   end
end

/* Global Var Cleanup */
drop ??FINDarray ??FINDindex ??FINDcase ??i ??THISline ??trash

return 1

/*********************/
SORTarray:
??FINDarray=arg(1)
??UPdown=arg(2)

if VERBOSE then say 'In SORTarray, ??FINDarray='??FINDarray' ??UPdown='??UPdown

/* Check Params */
if strip(??FINDarray)='' then do
   say 'SORTarray ERROR! -> Not Enough Paramaters'
   return 0
end

/* Verify UPdown param */
??UPdown=translate(??UPdown)
if ??UPdown\='A' & ??UPdown\='D' then ??UPdown='A'

/* define M for passes */
??m = 1
do while (9 * ??m + 4) < value(??FINDarray'.0')
   ??m = ??m * 3 + 1
end

/* One of Knuth's algorithms - sort the stem */
select
   when ??UPdown='A' then do while ??m > 0
      ??k = value(??FINDarray'.0') - ??m
      do ??j = 1 to ??k
         ??q = ??j
         do while ??q > 0
            ??l = ??q + ??m
            /* make comparisons case-independent */
            if translate(value(??FINDarray'.'??q)) <<= translate(value(??FINDarray'.'??l)) then leave
            /* switch elements               */
            ??tmp=value(??FINDarray'.'??q, value(??FINDarray'.'??l))
            ??trash=value(??FINDarray'.'??l, ??tmp)
            ??q = ??q - ??m
         end
      end
      ??m = ??m % 3
   end /* Ascending mode */

   when ??UPdown='D' then do while ??m > 0
      ??k = value(??FINDarray'.0') - ??m
      do ??j = 1 to ??k
         ??q = ??j
         do while ??q > 0
            ??l = ??q + ??m
            /* make comparisons case-independent */
            if translate(value(??FINDarray'.'??q)) >>= translate(value(??FINDarray'.'??l)) then leave
            /* switch elements               */
            ??tmp=value(??FINDarray'.'??q, value(??FINDarray'.'??l))
            ??trash=value(??FINDarray'.'??l, ??tmp)
            ??q = ??q - ??m
         end
      end
      ??m = ??m % 3
   end /* Decending mode */

end /* select */

/* Global Var Cleanup */
drop ??FINDarray ??UPdown ??m ??k ??j ??q ??l ??tmp ??trash

return 1

/*********************/
DELfile: procedure expose verbose
FINDfilename=arg(1)

if VERBOSE then say 'In DELfile, FINDfilename='FINDfilename

/* Check Params */
if strip(FINDfilename)='' then do
   say 'DELfile ERROR! -> Not Enough Paramaters'
   return 0
end

rc=RXDelete(FINDfilename)
if rc\=0 then do
   say 'DELfile -> File could not be deleted, RC='rc
   return 0
end

return 1

/*********************/
COPYfile: procedure expose verbose
FROMfilename=arg(1)
TOfilename=arg(2)

if VERBOSE then say 'In COPYfile, FROMfilename='FROMfilename' TOfilename='TOfilename

/* Check Params */
if strip(FROMfilename)='' | strip(TOfilename)='' then do
   say 'COPYfile ERROR! -> Not Enough Paramaters'
   return 0
end

/* Make sure the TOfilename does not exist so XCopy will work */
if ISfile(TOfilename) then do
   if \ DELfile(TOfilename) then do
      say 'COPYfile ERROR! -> Unable to modify 'TOfilename
      return 0
   end
end

/* Copy the file */
'echo f|xcopy 'FROMfilename' 'TOfilename' /v >nul'

if rc\=0 then do
   say 'COPYfile -> File could not be copied, RC='rc
   return 0
end

return 1

/*********************/
MAKEdir: procedure expose verbose
FINDfilename=arg(1)

if VERBOSE then say 'In MAKEdir, FINDfilename='FINDfilename

/* Check Params */
if strip(FINDfilename)='' then do
   say 'MAKEdir ERROR! -> Not Enough Paramaters'
   return 0
end

rc=RXMKDir(FINDfilename)
if rc\=0 then do
   say 'MAKEdir -> Dir could not be created, RC='rc
   return 0
end

return 1

/*********************/
DELdir: procedure expose verbose
FINDfilename=arg(1)

if VERBOSE then say 'In DELdir, FINDfilename='FINDfilename

/* Check Params */
if strip(FINDfilename)='' then do
   say 'DELdir ERROR! -> Not Enough Paramaters'
   return 0
end

rc=RXRMDir(FINDfilename)
if rc\=0 then do
   say 'DELdir -> Dir could not be deleted, RC='rc
   return 0
end

return 1

/*********************/
ISdir: procedure expose verbose
FINDfilename=arg(1)

if VERBOSE then say 'In ISdir, FINDfilename='FINDfilename

/* Check Params */
if strip(FINDfilename)='' then do
   say 'ISdir ERROR! -> Not Enough Paramaters'
   return 0
end

if RXISDir(FINDfilename)=1 then procrc=1
else procrc=0

return procrc

/*********************/
ISfile: procedure expose verbose
FINDfilename=arg(1)

if VERBOSE then say 'In ISfile, FINDfilename='FINDfilename

/* Check Params */
if strip(FINDfilename)='' then do
   say 'ISfile ERROR! -> Not Enough Paramaters'
   return 0
end

if RXISDir(FINDfilename)=0 then procrc=1
else procrc=0

return procrc

/*********************/
LDSFileSpec: procedure expose verbose
RUNmode=arg(1)
FINDfilename=arg(2)

if VERBOSE then say 'In LDSFileSpec, RUNmode='RUNmode' FINDfilename='FINDfilename

/* Check Params */
if strip(RUNmode)='' | strip(FINDfilename)='' then do
   say 'LDSFileSpec ERROR! -> Not Enough Paramaters'
   return 0
end

RUNmode=translate(strip(RUNmode))
FINDfilename=strip(FINDfilename)

/* See what slash we are using */
if pos('\', FINDfilename)\=0 & pos('/', FINDfilename)\=0 then do
   say 'LDSFileSpec ERROR! -> FINDfilename contains both slashes, can not process FINDfilename'
   return 0
end

if pos('\', FINDfilename)\=0 then do
   slash='\'
   doubleslash='\\'
end
else do
   slash='/'
   doubleslash='//'
end

select
   when RUNmode='DRIVE' then do
      FINDstring=left(FINDfilename, 2)
      select
         when datatype(left(FINDstring, 1), 'M') & right(FINDstring, 1)=':' then do
            /* DOS support, no extra code needed */
            nop
         end /* DOS */
         when FINDstring=doubleslash then do
            /* UNC mode */
            NEXTpos=pos(slash, right(FINDfilename, length(FINDfilename)-2))
            if NEXTpos=0 then do
               FINDstring=0
            end
            else do
               FINDstring=left(FINDfilename, NEXTpos+1)
            end
         end /* UNC */
         otherwise do
            FINDstring=0
         end
      end /* select */
   end

   when RUNmode='PATH' then do
      select
         when pos(slash, FINDfilename)\=0 & pos(doubleslash, FINDfilename)=0 then do
            /* DOS mode */
            FIRSTpos=pos(slash, FINDfilename)
            LASTpos=lastpos(slash, FINDfilename)
            if FIRSTpos=LASTpos then do
               FINDstring=0
            end
            else do
               FINDstring=substr(FINDfilename, FIRSTpos, LASTpos-FIRSTpos+1)
            end
         end /* DOS */
         when left(FINDfilename, 2)=doubleslash then do
            /* UNC mode */
            FINDfilename=right(FINDfilename, length(FINDfilename)-2)
            FIRSTpos=pos(slash, FINDfilename)
            LASTpos=lastpos(slash, FINDfilename)
            if FIRSTpos=LASTpos then do
               FINDstring=0
            end
            else do
               FINDstring=substr(FINDfilename, FIRSTpos, LASTpos-FIRSTpos+1)
            end
         end /* UNC */
         otherwise do
            FINDstring=0
         end
      end /* select */
   end

   when RUNmode='NAME' then do
      if pos(slash, FINDfilename)=0 then do
         FINDstring=0
      end
      else do
         FINDstring=right(FINDfilename, length(FINDfilename)-lastpos(slash, FINDfilename))
      end
   end

   otherwise do
      say 'LDSFileSpec ERROR! -> RUNmode='RUNmode' is not a supported RUNmode'
      FINDstring=0
   end
end /* select */

return FINDstring

/*********************/
Logger: procedure
LOGfile=arg(1)
LOGstring=arg(2)
TIMEstamp=arg(3)

/* Check Params */
if strip(LOGfile)='' | strip(LOGstring)='' then do
   say 'Logger ERROR! -> Not Enough Paramaters'
   return 0
end

/* Open a stream to read the file */
do i=1 to 8 until rc='READY'
   rc=stream(LOGfile,'C','OPEN WRITE')
end

/* Check that we got it open */
if rc\='READY' then return 0

/* Fast Forward to the end of the file */
rc=stream(LOGfile,'C','SEEK <'0)

/* Check to see if we need to TimeStamp the entry */
if TIMEstamp\=0 then LOGstring=date('U') || ' ' || time('C') || ' ' || LOGstring

/* Write out the log entry */
rc=LineOut(LOGfile,LOGstring)

/* Close the stream */
rc=stream(LOGfile,'C','CLOSE')

return 1

/*********************/
ISpctag: procedure expose verbose
PCTag=arg(1)

if VERBOSE then say 'In ISpctag, PCTag='PCTag

/* Check Params */
if strip(PCtag)='' then do
   say 'ISpctag ERROR! -> Not Enough Paramaters'
   return 0
end

PCTag=translate(PCTag)
PCTagLength=length(PCTag)

select
   when PCTagLength=5 then do
      if left(PCTag, 1)\='C' then return 0
      if datatype(right(PCTag, 4), 'N')\=1 then return 0
   end
   when PCTagLength=7 then do
      if left(PCTag, 2)\='PC' then return 0
      if datatype(right(PCTag, 5), 'N')\=1 then return 0
   end
   otherwise do
      if VERBOSE then say 'ISpctag -> Otherwise reached in Select statement'
      return 0
   end
end /* select */

return 1

/*********************/
SETUPcolor: proedure expose COLORarray. verbose

if VERBOSE then say 'In SETUPcolorarray'

drop COLORarray.
COLORarray.=0

/*------------------------------------------------------------------*/
/* Foreground colors:                                               */
/*------------------------------------------------------------------*/
COLORarray.FG.Black       = 0                     /* Black          */
COLORarray.FG.Red         = 4                     /* Red            */
COLORarray.FG.Green       = 2                     /* Green          */
COLORarray.FG.Yellow      = 6                     /* Yellow         */
COLORarray.FG.Blue        = 1                     /* Blue           */
COLORarray.FG.Magenta     = 5                     /* Magenta        */
COLORarray.FG.Cyan        = 3                     /* Cyan           */
COLORarray.FG.White       = 7                     /* White          */
COLORarray.FG.BoldBlack   = 8                     /* Bold Black     */
COLORarray.FG.BoldRed     = 12                    /* Bold Red       */
COLORarray.FG.BoldGreen   = 10                    /* Bold Green     */
COLORarray.FG.BoldYellow  = 14                    /* Bold Yellow    */
COLORarray.FG.BoldBlue    = 9                     /* Bold Blue      */
COLORarray.FG.BoldMagenta = 13                    /* Bold Magenta   */
COLORarray.FG.BoldCyan    = 11                    /* Bold Cyan      */
COLORarray.FG.BoldWhite   = 15                    /* Bold White     */

/*------------------------------------------------------------------*/
/* Background colors:                                               */
/*------------------------------------------------------------------*/
COLORarray.BG.Black       = 0                            /* Black   */
COLORarray.BG.Red         = 65                           /* Red     */
COLORarray.BG.Green       = 32                           /* Green   */
COLORarray.BG.Yellow      = 96                           /* Yellow  */
COLORarray.BG.Blue        = 16                           /* Blue    */
COLORarray.BG.Magenta     = 80                           /* Magenta */
COLORarray.BG.Cyan        = 48                           /* Cyan    */
COLORarray.BG.White       = 112                          /* White   */

/*------------------------------------------------------------------*/
/* Define Presets:                                                  */
/*------------------------------------------------------------------*/
/* All Colors on Black */
COLORarray.BlackOnBlack         = COLORarray.FG.Black       + COLORarray.BG.Black
COLORarray.RedOnBlack           = COLORarray.FG.Red         + COLORarray.BG.Black
COLORarray.GreenOnBlack         = COLORarray.FG.Green       + COLORarray.BG.Black
COLORarray.YellowOnBlack        = COLORarray.FG.Yellow      + COLORarray.BG.Black
COLORarray.BlueOnBlack          = COLORarray.FG.Blue        + COLORarray.BG.Black
COLORarray.MagentaOnBlack       = COLORarray.FG.Magenta     + COLORarray.BG.Black
COLORarray.CyanOnBlack          = COLORarray.FG.Cyan        + COLORarray.BG.Black
COLORarray.WhiteOnBlack         = COLORarray.FG.White       + COLORarray.BG.Black
COLORarray.BoldBlackOnBlack     = COLORarray.FG.BoldBlack   + COLORarray.BG.Black
COLORarray.BoldRedOnBlack       = COLORarray.FG.BoldRed     + COLORarray.BG.Black
COLORarray.BoldGreenOnBlack     = COLORarray.FG.BoldGreen   + COLORarray.BG.Black
COLORarray.BoldYellowOnBlack    = COLORarray.FG.BoldYellow  + COLORarray.BG.Black
COLORarray.BoldBlueOnBlack      = COLORarray.FG.BoldBlue    + COLORarray.BG.Black
COLORarray.BoldMagentaOnBlack   = COLORarray.FG.BoldMagenta + COLORarray.BG.Black
COLORarray.BoldCyanOnBlack      = COLORarray.FG.BoldCyan    + COLORarray.BG.Black
COLORarray.BoldWhiteOnBlack     = COLORarray.FG.BoldWhite   + COLORarray.BG.Black
/* All Colors on Red */
COLORarray.BlackOnRed           = COLORarray.FG.Black       + COLORarray.BG.Red
COLORarray.RedOnRed             = COLORarray.FG.Red         + COLORarray.BG.Red
COLORarray.GreenOnRed           = COLORarray.FG.Green       + COLORarray.BG.Red
COLORarray.YellowOnRed          = COLORarray.FG.Yellow      + COLORarray.BG.Red
COLORarray.BlueOnRed            = COLORarray.FG.Blue        + COLORarray.BG.Red
COLORarray.MagentaOnRed         = COLORarray.FG.Magenta     + COLORarray.BG.Red
COLORarray.CyanOnRed            = COLORarray.FG.Cyan        + COLORarray.BG.Red
COLORarray.WhiteOnRed           = COLORarray.FG.White       + COLORarray.BG.Red
COLORarray.BoldBlackOnRed       = COLORarray.FG.BoldBlack   + COLORarray.BG.Red
COLORarray.BoldRedOnRed         = COLORarray.FG.BoldRed     + COLORarray.BG.Red
COLORarray.BoldGreenOnRed       = COLORarray.FG.BoldGreen   + COLORarray.BG.Red
COLORarray.BoldYellowOnRed      = COLORarray.FG.BoldYellow  + COLORarray.BG.Red
COLORarray.BoldBlueOnRed        = COLORarray.FG.BoldBlue    + COLORarray.BG.Red
COLORarray.BoldMagentaOnRed     = COLORarray.FG.BoldMagenta + COLORarray.BG.Red
COLORarray.BoldCyanOnRed        = COLORarray.FG.BoldCyan    + COLORarray.BG.Red
COLORarray.BoldWhiteOnRed       = COLORarray.FG.BoldWhite   + COLORarray.BG.Red
/* All Colors on Green */
COLORarray.BlackOnGreen         = COLORarray.FG.Black       + COLORarray.BG.Green
COLORarray.RedOnGreen           = COLORarray.FG.Red         + COLORarray.BG.Green
COLORarray.GreenOnGreen         = COLORarray.FG.Green       + COLORarray.BG.Green
COLORarray.YellowOnGreen        = COLORarray.FG.Yellow      + COLORarray.BG.Green
COLORarray.BlueOnGreen          = COLORarray.FG.Blue        + COLORarray.BG.Green
COLORarray.MagentaOnGreen       = COLORarray.FG.Magenta     + COLORarray.BG.Green
COLORarray.CyanOnGreen          = COLORarray.FG.Cyan        + COLORarray.BG.Green
COLORarray.WhiteOnGreen         = COLORarray.FG.White       + COLORarray.BG.Green
COLORarray.BoldBlackOnGreen     = COLORarray.FG.BoldBlack   + COLORarray.BG.Green
COLORarray.BoldRedOnGreen       = COLORarray.FG.BoldRed     + COLORarray.BG.Green
COLORarray.BoldGreenOnGreen     = COLORarray.FG.BoldGreen   + COLORarray.BG.Green
COLORarray.BoldYellowOnGreen    = COLORarray.FG.BoldYellow  + COLORarray.BG.Green
COLORarray.BoldBlueOnGreen      = COLORarray.FG.BoldBlue    + COLORarray.BG.Green
COLORarray.BoldMagentaOnGreen   = COLORarray.FG.BoldMagenta + COLORarray.BG.Green
COLORarray.BoldCyanOnGreen      = COLORarray.FG.BoldCyan    + COLORarray.BG.Green
COLORarray.BoldWhiteOnGreen     = COLORarray.FG.BoldWhite   + COLORarray.BG.Green
/* All Colors on Yellow */
COLORarray.BlackOnYellow        = COLORarray.FG.Black       + COLORarray.BG.Yellow
COLORarray.RedOnYellow          = COLORarray.FG.Red         + COLORarray.BG.Yellow
COLORarray.GreenOnYellow        = COLORarray.FG.Green       + COLORarray.BG.Yellow
COLORarray.YellowOnYellow       = COLORarray.FG.Yellow      + COLORarray.BG.Yellow
COLORarray.BlueOnYellow         = COLORarray.FG.Blue        + COLORarray.BG.Yellow
COLORarray.MagentaOnYellow      = COLORarray.FG.Magenta     + COLORarray.BG.Yellow
COLORarray.CyanOnYellow         = COLORarray.FG.Cyan        + COLORarray.BG.Yellow
COLORarray.WhiteOnYellow        = COLORarray.FG.White       + COLORarray.BG.Yellow
COLORarray.BoldBlackOnYellow    = COLORarray.FG.BoldBlack   + COLORarray.BG.Yellow
COLORarray.BoldRedOnYellow      = COLORarray.FG.BoldRed     + COLORarray.BG.Yellow
COLORarray.BoldGreenOnYellow    = COLORarray.FG.BoldGreen   + COLORarray.BG.Yellow
COLORarray.BoldYellowOnYellow   = COLORarray.FG.BoldYellow  + COLORarray.BG.Yellow
COLORarray.BoldBlueOnYellow     = COLORarray.FG.BoldBlue    + COLORarray.BG.Yellow
COLORarray.BoldMagentaOnYellow  = COLORarray.FG.BoldMagenta + COLORarray.BG.Yellow
COLORarray.BoldCyanOnYellow     = COLORarray.FG.BoldCyan    + COLORarray.BG.Yellow
COLORarray.BoldWhiteOnYellow    = COLORarray.FG.BoldWhite   + COLORarray.BG.Yellow
/* All Colors on Blue */
COLORarray.BlackOnBlue          = COLORarray.FG.Black       + COLORarray.BG.Blue
COLORarray.RedOnBlue            = COLORarray.FG.Red         + COLORarray.BG.Blue
COLORarray.GreenOnBlue          = COLORarray.FG.Green       + COLORarray.BG.Blue
COLORarray.YellowOnBlue         = COLORarray.FG.Yellow      + COLORarray.BG.Blue
COLORarray.BlueOnBlue           = COLORarray.FG.Blue        + COLORarray.BG.Blue
COLORarray.MagentaOnBlue        = COLORarray.FG.Magenta     + COLORarray.BG.Blue
COLORarray.CyanOnBlue           = COLORarray.FG.Cyan        + COLORarray.BG.Blue
COLORarray.WhiteOnBlue          = COLORarray.FG.White       + COLORarray.BG.Blue
COLORarray.BoldBlackOnBlue      = COLORarray.FG.BoldBlack   + COLORarray.BG.Blue
COLORarray.BoldRedOnBlue        = COLORarray.FG.BoldRed     + COLORarray.BG.Blue
COLORarray.BoldGreenOnBlue      = COLORarray.FG.BoldGreen   + COLORarray.BG.Blue
COLORarray.BoldYellowOnBlue     = COLORarray.FG.BoldYellow  + COLORarray.BG.Blue
COLORarray.BoldBlueOnBlue       = COLORarray.FG.BoldBlue    + COLORarray.BG.Blue
COLORarray.BoldMagentaOnBlue    = COLORarray.FG.BoldMagenta + COLORarray.BG.Blue
COLORarray.BoldCyanOnBlue       = COLORarray.FG.BoldCyan    + COLORarray.BG.Blue
COLORarray.BoldWhiteOnBlue      = COLORarray.FG.BoldWhite   + COLORarray.BG.Blue
/* All Colors on Magenta */
COLORarray.BlackOnMagenta       = COLORarray.FG.Black       + COLORarray.BG.Magenta
COLORarray.RedOnMagenta         = COLORarray.FG.Red         + COLORarray.BG.Magenta
COLORarray.GreenOnMagenta       = COLORarray.FG.Green       + COLORarray.BG.Magenta
COLORarray.YellowOnMagenta      = COLORarray.FG.Yellow      + COLORarray.BG.Magenta
COLORarray.BlueOnMagenta        = COLORarray.FG.Blue        + COLORarray.BG.Magenta
COLORarray.MagentaOnMagenta     = COLORarray.FG.Magenta     + COLORarray.BG.Magenta
COLORarray.CyanOnMagenta        = COLORarray.FG.Cyan        + COLORarray.BG.Magenta
COLORarray.WhiteOnMagenta       = COLORarray.FG.White       + COLORarray.BG.Magenta
COLORarray.BoldBlackOnMagenta   = COLORarray.FG.BoldBlack   + COLORarray.BG.Magenta
COLORarray.BoldRedOnMagenta     = COLORarray.FG.BoldRed     + COLORarray.BG.Magenta
COLORarray.BoldGreenOnMagenta   = COLORarray.FG.BoldGreen   + COLORarray.BG.Magenta
COLORarray.BoldYellowOnMagenta  = COLORarray.FG.BoldYellow  + COLORarray.BG.Magenta
COLORarray.BoldBlueOnMagenta    = COLORarray.FG.BoldBlue    + COLORarray.BG.Magenta
COLORarray.BoldMagentaOnMagenta = COLORarray.FG.BoldMagenta + COLORarray.BG.Magenta
COLORarray.BoldCyanOnMagenta    = COLORarray.FG.BoldCyan    + COLORarray.BG.Magenta
COLORarray.BoldWhiteOnMagenta   = COLORarray.FG.BoldWhite   + COLORarray.BG.Magenta
/* All Colors on Cyan */
COLORarray.BlackOnCyan          = COLORarray.FG.Black       + COLORarray.BG.Cyan
COLORarray.RedOnCyan            = COLORarray.FG.Red         + COLORarray.BG.Cyan
COLORarray.GreenOnCyan          = COLORarray.FG.Green       + COLORarray.BG.Cyan
COLORarray.YellowOnCyan         = COLORarray.FG.Yellow      + COLORarray.BG.Cyan
COLORarray.BlueOnCyan           = COLORarray.FG.Blue        + COLORarray.BG.Cyan
COLORarray.MagentaOnCyan        = COLORarray.FG.Magenta     + COLORarray.BG.Cyan
COLORarray.CyanOnCyan           = COLORarray.FG.Cyan        + COLORarray.BG.Cyan
COLORarray.WhiteOnCyan          = COLORarray.FG.White       + COLORarray.BG.Cyan
COLORarray.BoldBlackOnCyan      = COLORarray.FG.BoldBlack   + COLORarray.BG.Cyan
COLORarray.BoldRedOnCyan        = COLORarray.FG.BoldRed     + COLORarray.BG.Cyan
COLORarray.BoldGreenOnCyan      = COLORarray.FG.BoldGreen   + COLORarray.BG.Cyan
COLORarray.BoldYellowOnCyan     = COLORarray.FG.BoldYellow  + COLORarray.BG.Cyan
COLORarray.BoldBlueOnCyan       = COLORarray.FG.BoldBlue    + COLORarray.BG.Cyan
COLORarray.BoldMagentaOnCyan    = COLORarray.FG.BoldMagenta + COLORarray.BG.Cyan
COLORarray.BoldCyanOnCyan       = COLORarray.FG.BoldCyan    + COLORarray.BG.Cyan
COLORarray.BoldWhiteOnCyan      = COLORarray.FG.BoldWhite   + COLORarray.BG.Cyan
/* All Colors on White */
COLORarray.BlackOnWhite         = COLORarray.FG.Black       + COLORarray.BG.White
COLORarray.RedOnWhite           = COLORarray.FG.Red         + COLORarray.BG.White
COLORarray.GreenOnWhite         = COLORarray.FG.Green       + COLORarray.BG.White
COLORarray.YellowOnWhite        = COLORarray.FG.Yellow      + COLORarray.BG.White
COLORarray.BlueOnWhite          = COLORarray.FG.Blue        + COLORarray.BG.White
COLORarray.MagentaOnWhite       = COLORarray.FG.Magenta     + COLORarray.BG.White
COLORarray.CyanOnWhite          = COLORarray.FG.Cyan        + COLORarray.BG.White
COLORarray.WhiteOnWhite         = COLORarray.FG.White       + COLORarray.BG.White
COLORarray.BoldBlackOnWhite     = COLORarray.FG.BoldBlack   + COLORarray.BG.White
COLORarray.BoldRedOnWhite       = COLORarray.FG.BoldRed     + COLORarray.BG.White
COLORarray.BoldGreenOnWhite     = COLORarray.FG.BoldGreen   + COLORarray.BG.White
COLORarray.BoldYellowOnWhite    = COLORarray.FG.BoldYellow  + COLORarray.BG.White
COLORarray.BoldBlueOnWhite      = COLORarray.FG.BoldBlue    + COLORarray.BG.White
COLORarray.BoldMagentaOnWhite   = COLORarray.FG.BoldMagenta + COLORarray.BG.White
COLORarray.BoldCyanOnWhite      = COLORarray.FG.BoldCyan    + COLORarray.BG.White
COLORarray.BoldWhiteOnWhite     = COLORarray.FG.BoldWhite   + COLORarray.BG.White

return 1

